# -*- coding: utf-8 -*-

import logging
import os

import six
from six.moves import configparser

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser

from shipane_sdk import Client
from shipane_sdk.joinquant.client import JoinQuantClient
from shipane_sdk.joinquant.runner import JoinQuantRunner

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, format='%(asctime)-15s %(levelname)-6s %(message)s')

    dir_path = os.path.dirname(os.path.realpath(__file__))

    config = ConfigParser()
    config.read('{}/config/config.ini'.format(dir_path))

    shipane_client = Client(host=config.get('ShiPanE', 'host'),
                            port=config.get('ShiPanE', 'port'),
                            key=config.get('ShiPanE', 'key'))
    jq_client = JoinQuantClient(username=config.get('JoinQuant', 'username'),
                                password=config.get('JoinQuant', 'password'),
                                backtest_id=config.get('JoinQuant', 'backtest_id'))
    jq_client.login()
    runner = JoinQuantRunner(shipane_client, jq_client,
                             interval=config.getint('JoinQuant', 'interval'),
                             idle_interval=config.getint('JoinQuant', 'idle_interval'))

    runner.run()



# -*- coding: utf-8 -*-

try:
    from shipane_sdk.client import Client
except:
    pass

try:
    from kuanke.user_space_api import *
except:
    pass


class JoinQuantExecutor(object):
    def __init__(self, **kwargs):
        if 'log' in globals():
            self._log = log
        else:
            import logging
            self._log = logging.getLogger()
        self._client = Client(**kwargs)
        self._order_id_map = dict()

    @property
    def client(self):
        return self._client

    @property
    def client(self):
        return self._client

    def execute(self, order):
        if order is None:
            self._log.info('[实盘易] 委托为空，忽略下单请求')
            return

        try:
            if order.is_buy:
                response = self._client.buy(order.security, order.price, order.amount)
            else:
                response = self._client.sell(order.security, order.price, order.amount)

            if response is not None:
                self._log.info(u'[实盘易] 回复如下：\nstatus_code: %d\ntext: %s', response.status_code, response.text)
            else:
                self._log.error('[实盘易] 未回复')

            if response is None:
                return None

            if response.status_code == 200:
                self._order_id_map[order.order_id] = response.json()['id'];

            return response
        except Exception as e:
            self._log.error("[实盘易] 下单异常：" + str(e))

    def cancel(self, order):
        if order is None:
            self._log.info('[实盘易] 委托为空，忽略撤单请求')
            return

        try:
            order_id = order if isinstance(order, int) else order.order_id
            if order_id in self._order_id_map:
                return self._client.cancel(self._order_id_map[order_id])
            else:
                self._log.warn('[实盘易] 未找到对应的委托编号')
        except Exception as e:
            self._log.error("[实盘易] 撤单异常：" + str(e))



# -*- coding: utf-8 -*-

import copy

import requests
from six.moves.urllib.parse import urlencode


class Client(object):
    def __init__(self, **kwargs):
        self._host = kwargs.pop('host', 'localhost')
        self._port = kwargs.pop('port', 8888)
        self._key = kwargs.pop('key', '')
        self._title = kwargs.pop('title', 'monijiaoyi')
        self._account = kwargs.pop('account', '')
        self._timeout = kwargs.pop('timeout', (5.0, 10.0))

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        self._host = value

    @property
    def port(self):
        return self._port

    @host.setter
    def port(self, value):
        self._port = value

    @property
    def key(self):
        return self._key

    @host.setter
    def key(self, value):
        self._key = value

    @property
    def title(self):
        return self._title

    @host.setter
    def title(self, value):
        self._title = value

    @property
    def account(self):
        return self._account

    @host.setter
    def account(self, value):
        self._account = value

    @property
    def timeout(self):
        return self._timeout

    @host.setter
    def timeout(self, value):
        self._timeout = value

    def get_account(self):
        return requests.get(self.__create_url('accounts'), timeout=self._timeout)

    def get_positions(self):
        return requests.get(self.__create_url('positions'), timeout=self._timeout)

    def buy(self, symbol, price, amount):
        return self.__execute('BUY', symbol, price, amount)

    def sell(self, symbol, price, amount):
        return self.__execute('SELL', symbol, price, amount)

    def execute(self, order_type, symbol, price, amount):
        return self.__execute(order_type, symbol, price, amount)

    def cancel(self, order_id):
        return requests.delete(self.__create_order_url(order_id), timeout=self._timeout)

    def cancel_all(self):
        return requests.delete(self.__create_order_url(), timeout=self._timeout)

    def query(self, navigation):
        return requests.get(self.__create_url('', navigation=navigation), timeout=self._timeout)

    def __execute(self, order_type, symbol, price, amount):
        return requests.post(self.__create_order_url(),
                             json={'orderType': order_type, 'symbol': symbol, 'price': price, 'amount': amount},
                             timeout=self._timeout)

    def __create_order_url(self, order_id=None, **params):
        return self.__create_url('orders', order_id, **params)

    def __create_url(self, resource, resource_id=None, **params):
        client_param = self.__create_client_param()
        all_params = copy.deepcopy(params)
        all_params.update(client=client_param, key=self._key)
        if resource_id is None:
            path = '/{}'.format(resource)
        else:
            path = '/{}/{}'.format(resource, resource_id)

        return '{}{}?{}'.format(self.__create_base_url(), path, urlencode(all_params))

    def __create_base_url(self):
        return 'http://' + self._host + ':' + str(self._port)

    def __create_client_param(self):
        client_param = ''
        if self._title:
            client_param += 'title:' + self._title
        if self._account:
            if client_param:
                client_param += ','
            client_param += 'account:' + self._account
        return client_param



# -*- coding: utf-8 -*-

import logging
import time
from datetime import datetime

from shipane_sdk.joinquant.transaction import JoinQuantTransaction
from shipane_sdk.market_utils import MarketUtils


class JoinQuantRunner(object):
    def __init__(self, shipane_client, jq_client, **kwargs):
        self._log = logging.getLogger()
        self._shipane_client = shipane_client
        self._jq_client = jq_client
        self._start_datatime = datetime.now()
        self._processed_transactions = []
        self._interval = kwargs.pop('interval', 10)
        self._idle_interval = kwargs.pop('idle_interval', 60)

    def run(self):
        while (True):
            if MarketUtils.is_closed():
                self._log.warn("********** 休市期间不跟单 **********")
                if self._processed_transactions:
                    del self._processed_transactions[:]

                time.sleep(self._idle_interval)
                continue

            self._log.info("********** 开始跟单 **********")

            try:
                transaction_detail = self._jq_client.query()
                raw_transactions = transaction_detail['data']['transaction']
                self._log.info("获取到 {} 条委托".format(len(raw_transactions)))

                transactions = []
                for raw_transaction in raw_transactions:
                    transaction = JoinQuantTransaction(raw_transaction).normalize()
                    if self._is_expired(transaction):
                        continue

                    transactions.append(transaction)
                self._log.info("获取到 {} 条有效委托".format(len(transactions)))

                for tx in transactions:
                    self._processed_transactions.append(tx)
                    self._log.info("开始以 {}元 {} {}股 {}".format(tx.price, tx.get_cn_type(), tx.amount, tx.symbol))
                    response = self._shipane_client.execute(tx.type, tx.symbol, tx.price, tx.amount)
                    if response is not None:
                        self._log.info(u'实盘易回复：\nstatus_code: %d\ntext: %s', response.status_code, response.text)
                    else:
                        self._log.error('实盘易未回复')
            except Exception as e:
                self._log.exception("跟单异常")

            self._log.info("********** 结束跟单 **********\n")
            time.sleep(self._interval)

    def _is_expired(self, transaction):
        if transaction.completed_at < self._start_datatime:
            return True
        if transaction in self._processed_transactions:
            return True
        return False



# -*- coding: utf-8 -*-

import inspect
import os
import unittest

import six
from six.moves import configparser

if six.PY2:
    ConfigParser = configparser.RawConfigParser
else:
    ConfigParser = configparser.ConfigParser

from shipane_sdk import Client


class ClientTest(unittest.TestCase):
    def setUp(self):
        config = ConfigParser()
        dir_path = os.path.dirname(os.path.realpath(__file__))
        config.read('{}/../config/config.ini'.format(dir_path))
        self.client = Client(host=config.get('ShiPanE', 'host'))

    def test_get_account(self):
        response = self.client.get_account()
        print(inspect.stack()[0][3] + ' - ' + response.text)
        self.assertEqual(response.status_code, 200)

    def test_get_positions(self):
        response = self.client.get_positions()
        print(inspect.stack()[0][3] + ' - ' + response.text)
        self.assertEqual(response.status_code, 200)

    def test_buy_stock(self):
        response = self.client.buy('000001', 8.11, 100)
        print(inspect.stack()[0][3] + ' - ' + response.text)
        json = response.json();
        if (response.status_code == 200):
            self.assertTrue(json['id'])
        elif (response.status_code == 400):
            self.assertTrue(json['message'])
        else:
            self.fail()

    def test_sell_stock(self):
        response = self.client.sell('000001', 9.51, 100)
        print(inspect.stack()[0][3] + ' - ' + response.text)
        json = response.json();
        if (response.status_code == 200):
            self.assertTrue(json['id'])
        elif (response.status_code == 400):
            self.assertTrue(json['message'])
        else:
            self.fail()

    def test_cancel_all(self):
        response = self.client.cancel_all()
        print(inspect.stack()[0][3] + ' - ' + response.text)
        self.assertEqual(response.status_code, 200)

    def test_query(self):
        response = self.client.query(navigation='查询>资金股份')
        print(inspect.stack()[0][3] + ' - ' + response.text)
        self.assertEqual(response.status_code, 200)
